<?php
# funkce

function mid_point($p1, $p2)
# Point mid_point (Point p1, Point p2)
# vrati bod, ktery je stredem usecky s koncovymi body $p1 a $p2
{
    return new Point(round(($p1->x+$p2->x)/2), round(($p1->y+$p2->y)/2));
}


function line_points($p1, $p2)
# array line_points (Point p1, Point p2)
# vrati pole souradnic vsech bodu, ktere lezi na spojnici bodu $p_1 a $p_2
{
    # rozdily souradnic
    $delta_x = abs($p2->x - $p1->x);
    $delta_y = abs($p2->y - $p1->y);
    
    # jestlize startovni a koncovy bod jsou sousednimi body, jsou jedinymi body
    #   usecky - vratit vysledne pole
    if ($delta_x <= 1 && $delta_y <=1)
    {
        return array($p1, $p2);
    }
    
    # jestlize zadane body nejsou sousedni, najit stred jejich spojnice, tim
    #   spojnici rozdelit na 2 poloviny a pro kazdou najit vsechny body, ktere
    #   na ni lezi
    $mid = mid_point($p1, $p2);
    # vysledky slijeme dohromady
    $points = array_merge(line_points($p1, $mid), line_points($mid, $p2));
    # a vyjmeme duplicitni
    $points = array_unique($points);
    
    return $points;
}


function find_path(&$parts_to_examine, &$path, $road)
# hleda cestu
{
    if (empty($parts_to_examine))
    {
        return;
    }
    
    $part = array_shift($parts_to_examine);
    
    # startovni bod useku
    $start_point = $part[0];
    # koncovy bod useku
    $end_point   = $part[1];
    # udrzeni stavu, zda jsme na trave
    $grass = false;
    
    foreach (line_points($start_point, $end_point) as $c_point)
    {
        # jestlize jsme prave z dosud prujezdneho useku vjeli na travu, zaznamenat tento bod
        if (!$grass && !$road[$c_point->x][$c_point->y])
        {
            $grass_start = $c_point;
            $grass = true;
        }
        
        # jestlize jsme prave vyjeli z travy, zaznamenat tento bod
        if ($grass && $road[$c_point->x][$c_point-y])
        {
            $grass_end = $c_point;
            break;
        }
    }

    # jestlize je usek prujezdny, ukoncit
    if (!$grass)
    {
        $path = array_merge(array($start_point), $path, array($end_point));
        return;
    }

    # stred 1. travnateho useku:
    $grass_mid = mid_point($grass_start, $grass_end);
    
    # urcime kolmici:
    # smernice puvodni spojnice
    $s1 = $end_point->x - $start_point->x;
    $s2 = $end_point->y - $start_point->y;
    # nova smernice - vektor kolmy
    $v1 = $s2;
    $v2 = -$s1;
    # normovat
    $v_coef = max(abs($v1), abs($v2));
    $v1 /= $v_coef;
    $v2 /= $v_coef;
    
    # projdeme vsechny body kolmice az k nejblizsimu, ktery je na silnici
    $nearest_point   = null;
    $nearest_point_2 = null;
    $t = -1;
    while (abs($t) < 255)
    {
        $x = $grass_mid->x + $v1*round($t);
        $y = $grass_mid->y + $v2*round($t);
        
        if ($road[$x][$y])
        {
            $nearest_point = new Point($x, $y);

            $sign = ($t >= 0 ? 1 : -1);
            $t = (abs($t) + 0.5) * $sign;
        }
        else
        {
            if (is_null($nearest_point))
            {
                $sign = ($t >= 0 ? 1 : -1);
                $t = (abs($t) + 0.5) * (-$sign);
            }
            else
            {
                # znovu jsme vyjeli na travu
                $nearest_point_2 = new Point($x, $y);
                break;
            }
        }
    }
    
    if (is_null($nearest_point_2))
    {
        $nearest_point_2 = new Point($x, $y);
    }
    
    if (is_null($nearest_point))
    {
        throw new PathException('Tra je neprjezdn.');
    }
    
    $A_point = mid_point($nearest_point, $nearest_point_2);
    # nyni hledejme cestu ze startu do A a z A do cile
    find_path(
        array_merge(
            $parts_to_examine, 
            array(array($start_point, $A_point)), 
            array(array($A_point, $end_point))
        ),
        $path,
        $road
    );

    
}


?>